# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/convert.ipynb.

# %% auto 0
__all__ = ['rename_filepath_to_match_datatype', 'change_suffix', 'ImageBlockConverter', 'md', 'convert_html_to_markdown',
           'convert_remove_html_tags', 'convert_enum_member_to_name', 'convert_str_to_snake_case',
           'convert_str_remove_accents', 'convert_str_keep_alphanumeric', 'convert_str_file_name',
           'convert_str_to_date', 'convert_datetime_to_str', 'sort_dictionary']

# %% ../../nbs/utils/convert.ipynb 3
from enum import Enum

import os
import re
import pathlib
import unicodedata


from markdownify import MarkdownConverter
import PIL
from bs4 import BeautifulSoup


import datetime as dt
from dateutil.parser import parse as dtu_parse

# %% ../../nbs/utils/convert.ipynb 6
def rename_filepath_to_match_datatype(data, file_path):
    is_path_ext = os.path.splitext(file_path)[-1].lower()

    old_suffix = pathlib.Path(file_path).suffix if is_path_ext else None

    new_suffix = ""

    if isinstance(data, str) or isinstance(data, bytes) or isinstance(data, bytearray):
        new_suffix = ".txt"
    if isinstance(data, dict):
        new_suffix = ".json"

    file_path = file_path + new_suffix

    if old_suffix:
        file_path = file_path.replace(old_suffix, "")

    return file_path

# %% ../../nbs/utils/convert.ipynb 8
def change_suffix(filename, new_extension):
    is_exist = os.path.exists(filename)

    p = pathlib.PurePath(filename)
    p = p.with_suffix(new_extension)

    if is_exist:
        p = pathlib.Path(filename)
        p = p.rename(p.with_suffix(new_extension))

    return str(p)

# %% ../../nbs/utils/convert.ipynb 11
class ImageBlockConverter(MarkdownConverter):
    """
    Create a custom MarkdownConverter that adds two newlines after an image
    """

    def convert_img(self, el, text, convert_as_inline, is_resize: bool = True):
        """
        custom image downloader for ImabeBlockConverter
        will handle resize
        """

        if is_resize:
            style_obj = {
                (obj.split(":")[0].strip()): obj.split(":")[1].strip()
                for obj in el.get("style").split(";")
                if ":" in obj
            }

            file_path = os.path.join(
                os.path.dirname(self.options["file_path"]), el["src"]
            )

            image = PIL.Image.open(file_path)

            width = style_obj["width"].replace("px", "")
            width = int(float(width))

            height = style_obj["height"].replace("px", "")
            height = int(float(height))

            new_image = image.resize((width, height))
            new_image.save(file_path)

        return super().convert_img(el, text, convert_as_inline)


def md(html, **options):
    """Create shorthand method for handling conversion"""
    return ImageBlockConverter(**options).convert(html)

# %% ../../nbs/utils/convert.ipynb 12
def convert_html_to_markdown(file_path):
    """converts html file to markdown in place"""

    with open(file_path, encoding="utf-8") as f:
        html = f.read()

    markdown_content = md(
        str(html),
        keep_inline_images_in=["td", "span"],
        file_path=file_path,
        is_resize=True,
    )

    md_path = file_path.replace(".html", ".md")

    with open(md_path, "w+", encoding="utf-8") as f:
        f.write(markdown_content)

    return

# %% ../../nbs/utils/convert.ipynb 13
def convert_remove_html_tags(html_string: str) -> str:
    """
    Clean an HTML string using BeautifulSoup.

    Args:
        html_string (str): The HTML string.

    Returns:
        str: The cleaned string.
    """
    soup = BeautifulSoup(html_string, "html.parser")
    clean_string = soup.get_text()
    return clean_string

# %% ../../nbs/utils/convert.ipynb 15
def convert_enum_member_to_name(enum_member):
    return enum_member.name if isinstance(enum_member, Enum) else enum_member

# %% ../../nbs/utils/convert.ipynb 16
def convert_str_to_snake_case(text_str):
    """converts 'snake_case_str' to 'snakeCaseStr'"""

    return text_str.replace(" ", "_").lower()

# %% ../../nbs/utils/convert.ipynb 17
def convert_str_remove_accents(text_str: str) -> str:
    return "".join(
        c
        for c in unicodedata.normalize("NFD", text_str)
        if unicodedata.category(c) != "Mn"
    )

# %% ../../nbs/utils/convert.ipynb 19
def convert_str_keep_alphanumeric(text_str) -> str:
    pattern = "[^0-9a-zA-Z_\-\s]+"

    return re.sub(pattern, "", text_str)

# %% ../../nbs/utils/convert.ipynb 20
def convert_str_file_name(text_str: str) -> str:
    """convert strings to clean file name or url"""

    return convert_str_keep_alphanumeric(
        convert_str_to_snake_case(convert_str_remove_accents(text_str))
    )

# %% ../../nbs/utils/convert.ipynb 22
def convert_str_to_date(datefield: str) -> dt.datetime:
    """converts string date to datetime object"""
    return dtu_parse(datefield) if datefield else None

# %% ../../nbs/utils/convert.ipynb 24
def convert_datetime_to_str(date_date: dt.datetime, format_str="%Y-%m-%d %H:%M") -> str:
    return date_date.strftime(format_str) if date_date else None

# %% ../../nbs/utils/convert.ipynb 25
def sort_dictionary(my_dict):
    my_keys = list(my_dict.keys())
    my_keys.sort()
    return {key: my_dict[key] for key in my_keys}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/pgp/pgp.ipynb.

# %% auto 0
__all__ = ['PGP_Key_ValidationError', 'PGP_Content_ValidationError', 'generate_pgpy_key', 'pgpy_encrypt', 'decrypt_pgp_file']

# %% ../nbs/pgp/pgp.ipynb 4
import datetime as dt
import json

import pgpy

import domolibrary_extensions.utils.utils as deut
import domolibrary_extensions.utils.convert as decv
import domolibrary_extensions.client as decl

# %% ../nbs/pgp/pgp.ipynb 6
class PGP_Key_ValidationError(decl.BaseError_Validation):
    def __init__(self, message):
        super().__init__(message)


class PGP_Content_ValidationError(decl.BaseError_Validation):
    def __init__(self, message):
        super().__init__(message)

# %% ../nbs/pgp/pgp.ipynb 7
def generate_pgpy_key(pgp_keyring_path=None, pgp_blob=None):
    pgp_key = None

    if not pgp_key and pgp_keyring_path:
        pgp_key, _ = pgpy.PGPKey.from_file(pgp_keyring_path)

    if not pgp_key and pgp_blob:
        pgp_key, _ = pgpy.PGPKey.from_blob(pgp_blob)

    if not pgp_key:
        raise PGP_Key_ValidationError("unable to generate pgpy key")

    return pgp_key

# %% ../nbs/pgp/pgp.ipynb 8
def pgpy_encrypt(
    data, pgp_keyring_path=None, pgp_blob=None, pgp_key=None, output_path: str = None
):
    pgp_key = pgp_key or generate_pgpy_key(
        pgp_keyring_path=pgp_keyring_path, pgp_blob=pgp_blob
    )

    if not pgp_key:
        raise PGP_Key_ValidationError("unable to generate pgpy key")

    if isinstance(data, dict):
        data = json.dumps(data)

    message = pgpy.PGPMessage.new(data)
    # message = pgpy.PGPMessage.new("This is a brand spankin' new message!")

    enc_message = bytes(pgp_key.pubkey.encrypt(message))

    if not output_path:
        return enc_message

    deut.upsert_folder(output_path)

    output_path = decv.change_suffix(filename=output_path, new_extension=".gpg")
    with open(output_path, "+wb") as f:
        f.write(enc_message)

    return f"output key to {output_path}"

# %% ../nbs/pgp/pgp.ipynb 10
def decrypt_pgp_file(
    pgp_password,
    pgp_keyring_path=None,
    pgp_blob=None,
    pgp_key=None,
    msg_blob=None,  # encrypted binary blob
    file_path=None,  # path to a encrypted file
    export_path=None,
):
    if not msg_blob and not file_path:
        raise PGP_Content_ValidationError(
            "must pass `msg_blob` or `file_path` to encrypted content"
        )

    pgp_key = pgp_key or generate_pgpy_key(
        pgp_keyring_path=pgp_keyring_path, pgp_blob=pgp_blob
    )

    if not pgp_key:
        raise PGP_Key_ValidationError("unable to generate pgpy key")

    emsg = None
    if file_path:
        emsg = pgpy.PGPMessage.from_file(file_path)

    if msg_blob:
        emsg = pgpy.PGPMessage.from_blob(msg_blob)

    obj = {
        "_decrypted_date": dt.datetime.now().strftime("%Y-%m-%d %H:%m"),
        "_file_path": file_path,
    }

    with pgp_key.unlock(pgp_password):
        data = pgp_key.decrypt(emsg).message

    try:
        data = json.loads(data)

    except ValueError as e:
        data = {"message": data}

    obj.update(**data)

    if not export_path:
        return obj

    export_path = decv.change_suffix(export_path, ".key")
    deut.upsert_folder(export_path)
    with open(export_path, "w+", encoding="utf-8") as f:
        f.write(json.dumps(obj))

    return f"key exported to {export_path}"



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/domolibrary_extensions',
                'doc_host': 'https://jaewilson07.github.io',
                'git_url': 'https://github.com/jaewilson07/domolibrary_extensions',
                'lib_path': 'domolibrary_extensions'},
  'syms': { 'domolibrary_extensions.asana.auth': { 'domolibrary_extensions.asana.auth.AsanaAuth': ( 'asana/auth.html#asanaauth',
                                                                                                    'domolibrary_extensions/asana/auth.py'),
                                                   'domolibrary_extensions.asana.auth.AsanaAuth.__post_init__': ( 'asana/auth.html#asanaauth.__post_init__',
                                                                                                                  'domolibrary_extensions/asana/auth.py'),
                                                   'domolibrary_extensions.asana.auth.AsanaAuth.generate_auth_header': ( 'asana/auth.html#asanaauth.generate_auth_header',
                                                                                                                         'domolibrary_extensions/asana/auth.py'),
                                                   'domolibrary_extensions.asana.auth.AsanaAuth.get_auth_token': ( 'asana/auth.html#asanaauth.get_auth_token',
                                                                                                                   'domolibrary_extensions/asana/auth.py')},
            'domolibrary_extensions.asana.project': { 'domolibrary_extensions.asana.project.AsanaProject': ( 'asana/project.html#asanaproject',
                                                                                                             'domolibrary_extensions/asana/project.py'),
                                                      'domolibrary_extensions.asana.project.AsanaProject._from_json': ( 'asana/project.html#asanaproject._from_json',
                                                                                                                        'domolibrary_extensions/asana/project.py'),
                                                      'domolibrary_extensions.asana.project.AsanaProject.get_by_id': ( 'asana/project.html#asanaproject.get_by_id',
                                                                                                                       'domolibrary_extensions/asana/project.py'),
                                                      'domolibrary_extensions.asana.project.AsanaProject.get_projects': ( 'asana/project.html#asanaproject.get_projects',
                                                                                                                          'domolibrary_extensions/asana/project.py')},
            'domolibrary_extensions.asana.task': { 'domolibrary_extensions.asana.task.AsanaMembership': ( 'asana/task.html#asanamembership',
                                                                                                          'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaMembership._from_json': ( 'asana/task.html#asanamembership._from_json',
                                                                                                                     'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaMembership.to_text': ( 'asana/task.html#asanamembership.to_text',
                                                                                                                  'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaSection': ( 'asana/task.html#asanasection',
                                                                                                       'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaSection._from_json': ( 'asana/task.html#asanasection._from_json',
                                                                                                                  'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaStory': ( 'asana/task.html#asanastory',
                                                                                                     'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaStory._from_json': ( 'asana/task.html#asanastory._from_json',
                                                                                                                'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaStory.to_text': ( 'asana/task.html#asanastory.to_text',
                                                                                                             'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaTask': ( 'asana/task.html#asanatask',
                                                                                                    'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaTask._from_json': ( 'asana/task.html#asanatask._from_json',
                                                                                                               'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaTask.get_by_id': ( 'asana/task.html#asanatask.get_by_id',
                                                                                                              'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaTask.get_stories': ( 'asana/task.html#asanatask.get_stories',
                                                                                                                'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.AsanaTask.to_md': ( 'asana/task.html#asanatask.to_md',
                                                                                                          'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.ap.AsanaProject.get_tasks': ( 'asana/task.html#ap.asanaproject.get_tasks',
                                                                                                                    'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.handle_render_datetime': ( 'asana/task.html#handle_render_datetime',
                                                                                                                 'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.handle_render_default': ( 'asana/task.html#handle_render_default',
                                                                                                                'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.handle_render_membership': ( 'asana/task.html#handle_render_membership',
                                                                                                                   'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.handle_render_stories': ( 'asana/task.html#handle_render_stories',
                                                                                                                'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.handle_render_user': ( 'asana/task.html#handle_render_user',
                                                                                                             'domolibrary_extensions/asana/task.py'),
                                                   'domolibrary_extensions.asana.task.render_field': ( 'asana/task.html#render_field',
                                                                                                       'domolibrary_extensions/asana/task.py')},
            'domolibrary_extensions.asana.user': { 'domolibrary_extensions.asana.user.AsanaUser': ( 'asana/user.html#asanauser',
                                                                                                    'domolibrary_extensions/asana/user.py'),
                                                   'domolibrary_extensions.asana.user.AsanaUser._from_json': ( 'asana/user.html#asanauser._from_json',
                                                                                                               'domolibrary_extensions/asana/user.py'),
                                                   'domolibrary_extensions.asana.user.AsanaUser.get_by_id': ( 'asana/user.html#asanauser.get_by_id',
                                                                                                              'domolibrary_extensions/asana/user.py')},
            'domolibrary_extensions.client': { 'domolibrary_extensions.client.Auth': ( 'client/client.html#auth',
                                                                                       'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.Auth.generate_auth_header': ( 'client/client.html#auth.generate_auth_header',
                                                                                                            'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.BaseError': ( 'client/client.html#baseerror',
                                                                                            'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.BaseError.__init__': ( 'client/client.html#baseerror.__init__',
                                                                                                     'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.BaseError_Validation': ( 'client/client.html#baseerror_validation',
                                                                                                       'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.BaseError_Validation.__init__': ( 'client/client.html#baseerror_validation.__init__',
                                                                                                                'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.ResponseGetData': ( 'client/client.html#responsegetdata',
                                                                                                  'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.ResponseGetData.__post_init__': ( 'client/client.html#responsegetdata.__post_init__',
                                                                                                                'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.ResponseGetData._from_cache': ( 'client/client.html#responsegetdata._from_cache',
                                                                                                              'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.ResponseGetData._from_httpx': ( 'client/client.html#responsegetdata._from_httpx',
                                                                                                              'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.ResponseGetData._from_stream': ( 'client/client.html#responsegetdata._from_stream',
                                                                                                               'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client._generate_cache_name': ( 'client/client.html#_generate_cache_name',
                                                                                                       'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.get_cache': ( 'client/client.html#get_cache',
                                                                                            'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.get_data': ( 'client/client.html#get_data',
                                                                                           'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.get_data_stream': ( 'client/client.html#get_data_stream',
                                                                                                  'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.looper': ( 'client/client.html#looper',
                                                                                         'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.prepare_fetch': ( 'client/client.html#prepare_fetch',
                                                                                                'domolibrary_extensions/client.py'),
                                               'domolibrary_extensions.client.update_cache': ( 'client/client.html#update_cache',
                                                                                               'domolibrary_extensions/client.py')},
            'domolibrary_extensions.google.GAuth': { 'domolibrary_extensions.google.GAuth.GAuth_Required': ( 'google/gauth.html#gauth_required',
                                                                                                             'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GAuth_Required.__init__': ( 'google/gauth.html#gauth_required.__init__',
                                                                                                                      'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GoogleAuth': ( 'google/gauth.html#googleauth',
                                                                                                         'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GoogleAuth.__post_init__': ( 'google/gauth.html#googleauth.__post_init__',
                                                                                                                       'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GoogleAuth.generate_from_env': ( 'google/gauth.html#googleauth.generate_from_env',
                                                                                                                           'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GoogleAuth.generate_from_file': ( 'google/gauth.html#googleauth.generate_from_file',
                                                                                                                            'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GoogleAuth.generate_service': ( 'google/gauth.html#googleauth.generate_service',
                                                                                                                          'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GoogleAuth_NoEnv_Error': ( 'google/gauth.html#googleauth_noenv_error',
                                                                                                                     'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.GoogleAuth_NoEnv_Error.__init__': ( 'google/gauth.html#googleauth_noenv_error.__init__',
                                                                                                                              'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth._output_creds': ( 'google/gauth.html#_output_creds',
                                                                                                            'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.generate_creds_from_env': ( 'google/gauth.html#generate_creds_from_env',
                                                                                                                      'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.generate_creds_from_file': ( 'google/gauth.html#generate_creds_from_file',
                                                                                                                       'domolibrary_extensions/google/GAuth.py'),
                                                     'domolibrary_extensions.google.GAuth.str_to_obj': ( 'google/gauth.html#str_to_obj',
                                                                                                         'domolibrary_extensions/google/GAuth.py')},
            'domolibrary_extensions.google.GDoc': { 'domolibrary_extensions.google.GDoc.DocState': ( 'google/gdoc.html#docstate',
                                                                                                     'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.DocState.run': ( 'google/gdoc.html#docstate.run',
                                                                                                         'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc': ( 'google/gdoc.html#gdoc',
                                                                                                 'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc.get_by_id': ( 'google/gdoc.html#gdoc.get_by_id',
                                                                                                           'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Content': ( 'google/gdoc.html#gdoc_content',
                                                                                                         'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Content._from_json': ( 'google/gdoc.html#gdoc_content._from_json',
                                                                                                                    'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Element': ( 'google/gdoc.html#gdoc_element',
                                                                                                         'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Element._from_json': ( 'google/gdoc.html#gdoc_element._from_json',
                                                                                                                    'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Element_TextRun': ( 'google/gdoc.html#gdoc_element_textrun',
                                                                                                                 'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Element_TextRun._from_json': ( 'google/gdoc.html#gdoc_element_textrun._from_json',
                                                                                                                            'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Paragraph': ( 'google/gdoc.html#gdoc_paragraph',
                                                                                                           'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Paragraph._from_json': ( 'google/gdoc.html#gdoc_paragraph._from_json',
                                                                                                                      'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Paragraph_Style': ( 'google/gdoc.html#gdoc_paragraph_style',
                                                                                                                 'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Paragraph_Style._from_json': ( 'google/gdoc.html#gdoc_paragraph_style._from_json',
                                                                                                                            'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_SectionBreak': ( 'google/gdoc.html#gdoc_sectionbreak',
                                                                                                              'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_SectionBreak_Style': ( 'google/gdoc.html#gdoc_sectionbreak_style',
                                                                                                                    'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_StyleEnum': ( 'google/gdoc.html#gdoc_styleenum',
                                                                                                           'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.GDoc_Text_Style': ( 'google/gdoc.html#gdoc_text_style',
                                                                                                            'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.generate_delete_json': ( 'google/gdoc.html#generate_delete_json',
                                                                                                                 'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.generate_insert_hyperlink': ( 'google/gdoc.html#generate_insert_hyperlink',
                                                                                                                      'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.generate_insert_json': ( 'google/gdoc.html#generate_insert_json',
                                                                                                                 'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.generate_update_style': ( 'google/gdoc.html#generate_update_style',
                                                                                                                  'domolibrary_extensions/google/GDoc.py'),
                                                    'domolibrary_extensions.google.GDoc.get_document_by_id': ( 'google/gdoc.html#get_document_by_id',
                                                                                                               'domolibrary_extensions/google/GDoc.py')},
            'domolibrary_extensions.google.GDrive': { 'domolibrary_extensions.google.GDrive.ExportAs_Enum': ( 'google/gdrive.html#exportas_enum',
                                                                                                              'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.ExportAs_Enum._missing_': ( 'google/gdrive.html#exportas_enum._missing_',
                                                                                                                        'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_ExportError': ( 'google/gdrive.html#gdrive_exporterror',
                                                                                                                   'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_ExportError.__init__': ( 'google/gdrive.html#gdrive_exporterror.__init__',
                                                                                                                            'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_File': ( 'google/gdrive.html#gdrive_file',
                                                                                                            'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_File.__post_init__': ( 'google/gdrive.html#gdrive_file.__post_init__',
                                                                                                                          'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_File._from_json': ( 'google/gdrive.html#gdrive_file._from_json',
                                                                                                                       'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_File.export': ( 'google/gdrive.html#gdrive_file.export',
                                                                                                                   'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_File.get_by_id': ( 'google/gdrive.html#gdrive_file.get_by_id',
                                                                                                                      'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_Files': ( 'google/gdrive.html#gdrive_files',
                                                                                                             'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_Files.__post_init__': ( 'google/gdrive.html#gdrive_files.__post_init__',
                                                                                                                           'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_Files.get_files': ( 'google/gdrive.html#gdrive_files.get_files',
                                                                                                                       'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_MimeType_Enum': ( 'google/gdrive.html#gdrive_mimetype_enum',
                                                                                                                     'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_ServiceRequired': ( 'google/gdrive.html#gdrive_servicerequired',
                                                                                                                       'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.GDrive_ServiceRequired.__init__': ( 'google/gdrive.html#gdrive_servicerequired.__init__',
                                                                                                                                'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.export_file_as': ( 'google/gdrive.html#export_file_as',
                                                                                                               'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.get_file_by_id': ( 'google/gdrive.html#get_file_by_id',
                                                                                                               'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.get_folder_by_id': ( 'google/gdrive.html#get_folder_by_id',
                                                                                                                 'domolibrary_extensions/google/GDrive.py'),
                                                      'domolibrary_extensions.google.GDrive.walk_folder': ( 'google/gdrive.html#walk_folder',
                                                                                                            'domolibrary_extensions/google/GDrive.py')},
            'domolibrary_extensions.google.GSearch': { 'domolibrary_extensions.google.GSearch.GoogleSearch': ( 'google/gsearch.html#googlesearch',
                                                                                                               'domolibrary_extensions/google/GSearch.py'),
                                                       'domolibrary_extensions.google.GSearch.GoogleSearch.search': ( 'google/gsearch.html#googlesearch.search',
                                                                                                                      'domolibrary_extensions/google/GSearch.py'),
                                                       'domolibrary_extensions.google.GSearch.GoogleSearchAuth': ( 'google/gsearch.html#googlesearchauth',
                                                                                                                   'domolibrary_extensions/google/GSearch.py'),
                                                       'domolibrary_extensions.google.GSearch.GoogleSearchAuth.__post_init__': ( 'google/gsearch.html#googlesearchauth.__post_init__',
                                                                                                                                 'domolibrary_extensions/google/GSearch.py'),
                                                       'domolibrary_extensions.google.GSearch.GoogleSearchAuth.generate_auth_header': ( 'google/gsearch.html#googlesearchauth.generate_auth_header',
                                                                                                                                        'domolibrary_extensions/google/GSearch.py')},
            'domolibrary_extensions.google.GSheet': { 'domolibrary_extensions.google.GSheet.GSheet': ( 'google/gsheet.html#gsheet',
                                                                                                       'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSheet._find_row': ( 'google/gsheet.html#gsheet._find_row',
                                                                                                                 'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSheet._from_json': ( 'google/gsheet.html#gsheet._from_json',
                                                                                                                  'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSheet._replace_row': ( 'google/gsheet.html#gsheet._replace_row',
                                                                                                                    'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSheet.append_rows': ( 'google/gsheet.html#gsheet.append_rows',
                                                                                                                   'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSheet.get_data': ( 'google/gsheet.html#gsheet.get_data',
                                                                                                                'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSheet.update_row': ( 'google/gsheet.html#gsheet.update_row',
                                                                                                                  'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSpreadsheet': ( 'google/gsheet.html#gspreadsheet',
                                                                                                             'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSpreadsheet.__post_init__': ( 'google/gsheet.html#gspreadsheet.__post_init__',
                                                                                                                           'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSpreadsheet.get_by_id': ( 'google/gsheet.html#gspreadsheet.get_by_id',
                                                                                                                       'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.GSpreadsheet.get_sheets': ( 'google/gsheet.html#gspreadsheet.get_sheets',
                                                                                                                        'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet._data_to_df': ( 'google/gsheet.html#_data_to_df',
                                                                                                            'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet._to_excel': ( 'google/gsheet.html#_to_excel',
                                                                                                          'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.append_rows': ( 'google/gsheet.html#append_rows',
                                                                                                            'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.find_row': ( 'google/gsheet.html#find_row',
                                                                                                         'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.get_sheet_values_df': ( 'google/gsheet.html#get_sheet_values_df',
                                                                                                                    'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.get_sheets': ( 'google/gsheet.html#get_sheets',
                                                                                                           'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.get_spreadsheet_by_id': ( 'google/gsheet.html#get_spreadsheet_by_id',
                                                                                                                      'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.insert_rows': ( 'google/gsheet.html#insert_rows',
                                                                                                            'domolibrary_extensions/google/GSheet.py'),
                                                      'domolibrary_extensions.google.GSheet.replace_row': ( 'google/gsheet.html#replace_row',
                                                                                                            'domolibrary_extensions/google/GSheet.py')},
            'domolibrary_extensions.integrations.ProxyCurl': { 'domolibrary_extensions.integrations.ProxyCurl.ProxyCurlAuth': ( 'integrations/proxycurl.html#proxycurlauth',
                                                                                                                                'domolibrary_extensions/integrations/ProxyCurl.py'),
                                                               'domolibrary_extensions.integrations.ProxyCurl.ProxyCurlAuth.generate_auth_header': ( 'integrations/proxycurl.html#proxycurlauth.generate_auth_header',
                                                                                                                                                     'domolibrary_extensions/integrations/ProxyCurl.py'),
                                                               'domolibrary_extensions.integrations.ProxyCurl.get_linked_in_profile_route': ( 'integrations/proxycurl.html#get_linked_in_profile_route',
                                                                                                                                              'domolibrary_extensions/integrations/ProxyCurl.py'),
                                                               'domolibrary_extensions.integrations.ProxyCurl.scrape_linkedin_profile': ( 'integrations/proxycurl.html#scrape_linkedin_profile',
                                                                                                                                          'domolibrary_extensions/integrations/ProxyCurl.py')},
            'domolibrary_extensions.jira.auth': { 'domolibrary_extensions.jira.auth.JiraAuth': ( 'jira/auth.html#jiraauth',
                                                                                                 'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth.JiraAuthPat': ( 'jira/auth.html#jiraauthpat',
                                                                                                    'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth.JiraAuthPat.__post_init__': ( 'jira/auth.html#jiraauthpat.__post_init__',
                                                                                                                  'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth.JiraAuthPat.generate_auth_header': ( 'jira/auth.html#jiraauthpat.generate_auth_header',
                                                                                                                         'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth.JiraAuthPat.who_am_i': ( 'jira/auth.html#jiraauthpat.who_am_i',
                                                                                                             'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth._JiraAuthPat_Required': ( 'jira/auth.html#_jiraauthpat_required',
                                                                                                              'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth._JiraAuthPat_Required.__post_init__': ( 'jira/auth.html#_jiraauthpat_required.__post_init__',
                                                                                                                            'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth._JiraAuth_Optional': ( 'jira/auth.html#_jiraauth_optional',
                                                                                                           'domolibrary_extensions/jira/auth.py'),
                                                  'domolibrary_extensions.jira.auth._JiraAuth_Required': ( 'jira/auth.html#_jiraauth_required',
                                                                                                           'domolibrary_extensions/jira/auth.py')},
            'domolibrary_extensions.jira.classes': { 'domolibrary_extensions.jira.classes.JiraComment': ( 'jira/classes.html#jiracomment',
                                                                                                          'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraComment.from_json': ( 'jira/classes.html#jiracomment.from_json',
                                                                                                                    'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraIssue': ( 'jira/classes.html#jiraissue',
                                                                                                        'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraIssue.from_json': ( 'jira/classes.html#jiraissue.from_json',
                                                                                                                  'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraIssue.get_by_id': ( 'jira/classes.html#jiraissue.get_by_id',
                                                                                                                  'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraIssue.get_comments': ( 'jira/classes.html#jiraissue.get_comments',
                                                                                                                     'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraIssue.get_links': ( 'jira/classes.html#jiraissue.get_links',
                                                                                                                  'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraIssueType': ( 'jira/classes.html#jiraissuetype',
                                                                                                            'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraIssueType.from_json': ( 'jira/classes.html#jiraissuetype.from_json',
                                                                                                                      'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraPerson': ( 'jira/classes.html#jiraperson',
                                                                                                         'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraPerson.from_json': ( 'jira/classes.html#jiraperson.from_json',
                                                                                                                   'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraPriority': ( 'jira/classes.html#jirapriority',
                                                                                                           'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraPriority.from_json': ( 'jira/classes.html#jirapriority.from_json',
                                                                                                                     'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraProject': ( 'jira/classes.html#jiraproject',
                                                                                                          'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraProject.from_json': ( 'jira/classes.html#jiraproject.from_json',
                                                                                                                    'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraRemoteLink': ( 'jira/classes.html#jiraremotelink',
                                                                                                             'domolibrary_extensions/jira/classes.py'),
                                                     'domolibrary_extensions.jira.classes.JiraRemoteLink.from_json': ( 'jira/classes.html#jiraremotelink.from_json',
                                                                                                                       'domolibrary_extensions/jira/classes.py')},
            'domolibrary_extensions.jira.routes': { 'domolibrary_extensions.jira.routes.get_board_epics': ( 'jira/routes.html#get_board_epics',
                                                                                                            'domolibrary_extensions/jira/routes.py'),
                                                    'domolibrary_extensions.jira.routes.get_board_issues': ( 'jira/routes.html#get_board_issues',
                                                                                                             'domolibrary_extensions/jira/routes.py'),
                                                    'domolibrary_extensions.jira.routes.get_boards': ( 'jira/routes.html#get_boards',
                                                                                                       'domolibrary_extensions/jira/routes.py'),
                                                    'domolibrary_extensions.jira.routes.get_epic': ( 'jira/routes.html#get_epic',
                                                                                                     'domolibrary_extensions/jira/routes.py'),
                                                    'domolibrary_extensions.jira.routes.get_issue_by_id': ( 'jira/routes.html#get_issue_by_id',
                                                                                                            'domolibrary_extensions/jira/routes.py'),
                                                    'domolibrary_extensions.jira.routes.get_issue_comments': ( 'jira/routes.html#get_issue_comments',
                                                                                                               'domolibrary_extensions/jira/routes.py'),
                                                    'domolibrary_extensions.jira.routes.get_issue_links': ( 'jira/routes.html#get_issue_links',
                                                                                                            'domolibrary_extensions/jira/routes.py')},
            'domolibrary_extensions.onepass': { 'domolibrary_extensions.onepass.Item_Field': ( '1pass/op.html#item_field',
                                                                                               'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Field.__eq__': ( '1pass/op.html#item_field.__eq__',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Field.__repr__': ( '1pass/op.html#item_field.__repr__',
                                                                                                        'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Field._from_sdk': ( '1pass/op.html#item_field._from_sdk',
                                                                                                         'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Field.generate_recipie': ( '1pass/op.html#item_field.generate_recipie',
                                                                                                                'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Field.to_sdk': ( '1pass/op.html#item_field.to_sdk',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Section': ( '1pass/op.html#item_section',
                                                                                                 'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Section._from_sdk': ( '1pass/op.html#item_section._from_sdk',
                                                                                                           'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Section.to_sdk': ( '1pass/op.html#item_section.to_sdk',
                                                                                                        'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Item_Section.to_sdk_for_item': ( '1pass/op.html#item_section.to_sdk_for_item',
                                                                                                                 'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.KnownItemPurpose': ( '1pass/op.html#knownitempurpose',
                                                                                                     'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.KnownItemType': ( '1pass/op.html#knownitemtype',
                                                                                                  'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.PreferredItemCategory': ( '1pass/op.html#preferreditemcategory',
                                                                                                          'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault': ( '1pass/op.html#vault',
                                                                                          'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault.create_item': ( '1pass/op.html#vault.create_item',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault.from_sdk': ( '1pass/op.html#vault.from_sdk',
                                                                                                   'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault.get_item': ( '1pass/op.html#vault.get_item',
                                                                                                   'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault.get_items': ( '1pass/op.html#vault.get_items',
                                                                                                    'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault.get_vault': ( '1pass/op.html#vault.get_vault',
                                                                                                    'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault.upsert_item': ( '1pass/op.html#vault.upsert_item',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.VaultItem_GET_Excception': ( '1pass/op.html#vaultitem_get_excception',
                                                                                                             'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.VaultItem_GET_Excception.__init__': ( '1pass/op.html#vaultitem_get_excception.__init__',
                                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item': ( '1pass/op.html#vault_item',
                                                                                               'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item.__eq__': ( '1pass/op.html#vault_item.__eq__',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item._from_sdk': ( '1pass/op.html#vault_item._from_sdk',
                                                                                                         'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item.add_field': ( '1pass/op.html#vault_item.add_field',
                                                                                                         'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item.delete': ( '1pass/op.html#vault_item.delete',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item.get_by_id': ( '1pass/op.html#vault_item.get_by_id',
                                                                                                         'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item.to_sdk': ( '1pass/op.html#vault_item.to_sdk',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.Vault_Item.update': ( '1pass/op.html#vault_item.update',
                                                                                                      'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.clean_dict': ( '1pass/op.html#clean_dict',
                                                                                               'domolibrary_extensions/onepass.py'),
                                                'domolibrary_extensions.onepass.generate_password_recipe': ( '1pass/op.html#generate_password_recipe',
                                                                                                             'domolibrary_extensions/onepass.py')},
            'domolibrary_extensions.pgp': { 'domolibrary_extensions.pgp.PGP_Content_ValidationError': ( 'pgp/pgp.html#pgp_content_validationerror',
                                                                                                        'domolibrary_extensions/pgp.py'),
                                            'domolibrary_extensions.pgp.PGP_Content_ValidationError.__init__': ( 'pgp/pgp.html#pgp_content_validationerror.__init__',
                                                                                                                 'domolibrary_extensions/pgp.py'),
                                            'domolibrary_extensions.pgp.PGP_Key_ValidationError': ( 'pgp/pgp.html#pgp_key_validationerror',
                                                                                                    'domolibrary_extensions/pgp.py'),
                                            'domolibrary_extensions.pgp.PGP_Key_ValidationError.__init__': ( 'pgp/pgp.html#pgp_key_validationerror.__init__',
                                                                                                             'domolibrary_extensions/pgp.py'),
                                            'domolibrary_extensions.pgp.decrypt_pgp_file': ( 'pgp/pgp.html#decrypt_pgp_file',
                                                                                             'domolibrary_extensions/pgp.py'),
                                            'domolibrary_extensions.pgp.generate_pgpy_key': ( 'pgp/pgp.html#generate_pgpy_key',
                                                                                              'domolibrary_extensions/pgp.py'),
                                            'domolibrary_extensions.pgp.pgpy_encrypt': ( 'pgp/pgp.html#pgpy_encrypt',
                                                                                         'domolibrary_extensions/pgp.py')},
            'domolibrary_extensions.utils.Image': { 'domolibrary_extensions.utils.Image.Image.from_bytestr': ( 'utils/image.html#image.from_bytestr',
                                                                                                               'domolibrary_extensions/utils/Image.py'),
                                                    'domolibrary_extensions.utils.Image.Image.from_image_file': ( 'utils/image.html#image.from_image_file',
                                                                                                                  'domolibrary_extensions/utils/Image.py'),
                                                    'domolibrary_extensions.utils.Image.are_same_image': ( 'utils/image.html#are_same_image',
                                                                                                           'domolibrary_extensions/utils/Image.py'),
                                                    'domolibrary_extensions.utils.Image.crop_square': ( 'utils/image.html#crop_square',
                                                                                                        'domolibrary_extensions/utils/Image.py'),
                                                    'domolibrary_extensions.utils.Image.handle_string_to_bytes_and_decode': ( 'utils/image.html#handle_string_to_bytes_and_decode',
                                                                                                                              'domolibrary_extensions/utils/Image.py'),
                                                    'domolibrary_extensions.utils.Image.handle_string_to_bytes_and_encode': ( 'utils/image.html#handle_string_to_bytes_and_encode',
                                                                                                                              'domolibrary_extensions/utils/Image.py'),
                                                    'domolibrary_extensions.utils.Image.isBase64': ( 'utils/image.html#isbase64',
                                                                                                     'domolibrary_extensions/utils/Image.py'),
                                                    'domolibrary_extensions.utils.Image.to_bytes': ( 'utils/image.html#to_bytes',
                                                                                                     'domolibrary_extensions/utils/Image.py')},
            'domolibrary_extensions.utils.convert': { 'domolibrary_extensions.utils.convert.ImageBlockConverter': ( 'utils/convert.html#imageblockconverter',
                                                                                                                    'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.ImageBlockConverter.convert_img': ( 'utils/convert.html#imageblockconverter.convert_img',
                                                                                                                                'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.change_suffix': ( 'utils/convert.html#change_suffix',
                                                                                                              'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_datetime_to_str': ( 'utils/convert.html#convert_datetime_to_str',
                                                                                                                        'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_enum_member_to_name': ( 'utils/convert.html#convert_enum_member_to_name',
                                                                                                                            'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_html_to_markdown': ( 'utils/convert.html#convert_html_to_markdown',
                                                                                                                         'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_remove_html_tags': ( 'utils/convert.html#convert_remove_html_tags',
                                                                                                                         'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_str_file_name': ( 'utils/convert.html#convert_str_file_name',
                                                                                                                      'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_str_keep_alphanumeric': ( 'utils/convert.html#convert_str_keep_alphanumeric',
                                                                                                                              'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_str_remove_accents': ( 'utils/convert.html#convert_str_remove_accents',
                                                                                                                           'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_str_to_date': ( 'utils/convert.html#convert_str_to_date',
                                                                                                                    'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.convert_str_to_snake_case': ( 'utils/convert.html#convert_str_to_snake_case',
                                                                                                                          'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.md': ( 'utils/convert.html#md',
                                                                                                   'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.rename_filepath_to_match_datatype': ( 'utils/convert.html#rename_filepath_to_match_datatype',
                                                                                                                                  'domolibrary_extensions/utils/convert.py'),
                                                      'domolibrary_extensions.utils.convert.sort_dictionary': ( 'utils/convert.html#sort_dictionary',
                                                                                                                'domolibrary_extensions/utils/convert.py')},
            'domolibrary_extensions.utils.execution': { 'domolibrary_extensions.utils.execution.gather_with_concurrency': ( 'utils/execution.html#gather_with_concurrency',
                                                                                                                            'domolibrary_extensions/utils/execution.py')},
            'domolibrary_extensions.utils.factory': { 'domolibrary_extensions.utils.factory.FactoryConfig': ( 'factory/factory.html#factoryconfig',
                                                                                                              'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryConfig.run': ( 'factory/factory.html#factoryconfig.run',
                                                                                                                  'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_Error': ( 'factory/factory.html#factoryfunction_error',
                                                                                                                      'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_Error.__init__': ( 'factory/factory.html#factoryfunction_error.__init__',
                                                                                                                               'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_MissingParameter': ( 'factory/factory.html#factoryfunction_missingparameter',
                                                                                                                                 'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_MissingParameter.__init__': ( 'factory/factory.html#factoryfunction_missingparameter.__init__',
                                                                                                                                          'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_NotSuccess': ( 'factory/factory.html#factoryfunction_notsuccess',
                                                                                                                           'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_NotSuccess.__init__': ( 'factory/factory.html#factoryfunction_notsuccess.__init__',
                                                                                                                                    'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_ResponseError': ( 'factory/factory.html#factoryfunction_responseerror',
                                                                                                                              'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_ResponseError.__init__': ( 'factory/factory.html#factoryfunction_responseerror.__init__',
                                                                                                                                       'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_ResponseTypeError': ( 'factory/factory.html#factoryfunction_responsetypeerror',
                                                                                                                                  'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryFunction_ResponseTypeError.__init__': ( 'factory/factory.html#factoryfunction_responsetypeerror.__init__',
                                                                                                                                           'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryLogs': ( 'factory/factory.html#factorylogs',
                                                                                                            'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryLogs.add_response': ( 'factory/factory.html#factorylogs.add_response',
                                                                                                                         'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryLogs.to_json': ( 'factory/factory.html#factorylogs.to_json',
                                                                                                                    'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryMessage': ( 'factory/factory.html#factorymessage',
                                                                                                               'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryMessage.__eq__': ( 'factory/factory.html#factorymessage.__eq__',
                                                                                                                      'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryMessage.to_json': ( 'factory/factory.html#factorymessage.to_json',
                                                                                                                       'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryMessage.to_string': ( 'factory/factory.html#factorymessage.to_string',
                                                                                                                         'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryResponse': ( 'factory/factory.html#factoryresponse',
                                                                                                                'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryResponse.__eq__': ( 'factory/factory.html#factoryresponse.__eq__',
                                                                                                                       'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryResponse.add_message': ( 'factory/factory.html#factoryresponse.add_message',
                                                                                                                            'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryResponse.test_success': ( 'factory/factory.html#factoryresponse.test_success',
                                                                                                                             'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.FactoryResponse.to_json': ( 'factory/factory.html#factoryresponse.to_json',
                                                                                                                        'domolibrary_extensions/utils/factory.py'),
                                                      'domolibrary_extensions.utils.factory.factory_function': ( 'factory/factory.html#factory_function',
                                                                                                                 'domolibrary_extensions/utils/factory.py')},
            'domolibrary_extensions.utils.utils': { 'domolibrary_extensions.utils.utils.ImageBlockConverter': ( 'utils/utils.html#imageblockconverter',
                                                                                                                'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.ImageBlockConverter.convert_img': ( 'utils/utils.html#imageblockconverter.convert_img',
                                                                                                                            'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.convert_enum_member_to_name': ( 'utils/utils.html#convert_enum_member_to_name',
                                                                                                                        'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.convert_html_to_markdown': ( 'utils/utils.html#convert_html_to_markdown',
                                                                                                                     'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.convert_str_file_name': ( 'utils/utils.html#convert_str_file_name',
                                                                                                                  'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.convert_str_keep_alphanumeric': ( 'utils/utils.html#convert_str_keep_alphanumeric',
                                                                                                                          'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.convert_str_remove_accents': ( 'utils/utils.html#convert_str_remove_accents',
                                                                                                                       'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.convert_str_to_date': ( 'utils/utils.html#convert_str_to_date',
                                                                                                                'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.convert_str_to_snake_case': ( 'utils/utils.html#convert_str_to_snake_case',
                                                                                                                      'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.detect_encoding': ( 'utils/utils.html#detect_encoding',
                                                                                                            'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.download_pptx': ( 'utils/utils.html#download_pptx',
                                                                                                          'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.download_zip': ( 'utils/utils.html#download_zip',
                                                                                                         'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.get_all_files_and_folders': ( 'utils/utils.html#get_all_files_and_folders',
                                                                                                                      'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.md': ( 'utils/utils.html#md',
                                                                                               'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.read_html_file': ( 'utils/utils.html#read_html_file',
                                                                                                           'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.remove_query_params_from_url': ( 'utils/utils.html#remove_query_params_from_url',
                                                                                                                         'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.rename_filepath_to_match_datatype': ( 'utils/utils.html#rename_filepath_to_match_datatype',
                                                                                                                              'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.update_env': ( 'utils/utils.html#update_env',
                                                                                                       'domolibrary_extensions/utils/utils.py'),
                                                    'domolibrary_extensions.utils.utils.upsert_folder': ( 'utils/utils.html#upsert_folder',
                                                                                                          'domolibrary_extensions/utils/utils.py')},
            'domolibrary_extensions.vanilla.Vanilla': { 'domolibrary_extensions.vanilla.Vanilla.Attribute': ( 'vanilla/classes.html#attribute',
                                                                                                              'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Breadcrumb': ( 'vanilla/classes.html#breadcrumb',
                                                                                                               'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Breadcrumb._from_json': ( 'vanilla/classes.html#breadcrumb._from_json',
                                                                                                                          'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Category': ( 'vanilla/classes.html#category',
                                                                                                             'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Category._from_json': ( 'vanilla/classes.html#category._from_json',
                                                                                                                        'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Category.get_by_id': ( 'vanilla/classes.html#category.get_by_id',
                                                                                                                       'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Discussion': ( 'vanilla/classes.html#discussion',
                                                                                                               'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Discussion._attributes_from_json': ( 'vanilla/classes.html#discussion._attributes_from_json',
                                                                                                                                     'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Discussion._from_json': ( 'vanilla/classes.html#discussion._from_json',
                                                                                                                          'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Discussion.to_dict': ( 'vanilla/classes.html#discussion.to_dict',
                                                                                                                       'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Image': ( 'vanilla/classes.html#image',
                                                                                                          'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Question': ( 'vanilla/classes.html#question',
                                                                                                             'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.Question._from_json': ( 'vanilla/classes.html#question._from_json',
                                                                                                                        'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.QuestionStatusEnum': ( 'vanilla/classes.html#questionstatusenum',
                                                                                                                       'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.UrlSrcSet': ( 'vanilla/classes.html#urlsrcset',
                                                                                                              'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.UrlSrcSet._from_json': ( 'vanilla/classes.html#urlsrcset._from_json',
                                                                                                                         'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.User': ( 'vanilla/classes.html#user',
                                                                                                         'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.User._from_json': ( 'vanilla/classes.html#user._from_json',
                                                                                                                    'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.VanillaForums': ( 'vanilla/classes.html#vanillaforums',
                                                                                                                  'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.VanillaForums.generate_discussion_report': ( 'vanilla/classes.html#vanillaforums.generate_discussion_report',
                                                                                                                                             'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.VanillaForums.get_all_categories': ( 'vanilla/classes.html#vanillaforums.get_all_categories',
                                                                                                                                     'domolibrary_extensions/vanilla/Vanilla.py'),
                                                        'domolibrary_extensions.vanilla.Vanilla.VanillaForums.get_all_discussions': ( 'vanilla/classes.html#vanillaforums.get_all_discussions',
                                                                                                                                      'domolibrary_extensions/vanilla/Vanilla.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/jira/routes.ipynb.

# %% auto 0
__all__ = ['get_boards', 'get_board_epics', 'get_epic', 'get_issue_by_id', 'get_board_issues', 'get_issue_links',
           'get_issue_comments']

# %% ../../nbs/jira/routes.ipynb 2
import httpx

import domolibrary_extensions.client as gd
import domolibrary_extensions.jira.auth as ja

# %% ../../nbs/jira/routes.ipynb 5
async def get_boards(
    auth: ja.JiraAuth,
    params: dict = None,
    debug_api: bool = False,
    debug_loop: bool = False,
    return_raw: bool = False,
    client: httpx.AsyncClient = None,
    is_ignore_cache: bool = False,
    cache_path: str = None,
    timeout: int = 30,
):
    url = f"https://{auth.instance}/rest/agile/1.0/board"

    def arr_fn(res):
        return res.response["values"]

    client = client or httpx.AsyncClient()

    res = await gd.looper(
        url=url,
        auth=auth,
        arr_fn=arr_fn,
        params=params,
        method="GET",
        debug_loop=debug_loop,
        debug_api=debug_api,
        client=client,
        is_ignore_cache=is_ignore_cache,
        cache_path=cache_path,
        timeout=timeout,
    )

    if return_raw:
        return res

    return res.response

# %% ../../nbs/jira/routes.ipynb 11
async def get_board_epics(
    auth: ja.JiraAuth,
    board_id: int,
    params: dict = None,
    debug_api: bool = False,
    debug_loop: bool = False,
    return_raw: bool = False,
    is_ignore_cache: bool = False,
    cache_path: str = None,
    client: httpx.AsyncClient = None,
    timeout: int = 30,
):
    url = f"https://{auth.instance}/rest/agile/1.0/board/{board_id}/epic"

    def arr_fn(res):
        return res.response["values"]

    res = await gd.looper(
        method="get",
        url=url,
        auth=auth,
        params=params,
        debug_api=debug_api,
        debug_loop=debug_loop,
        arr_fn=arr_fn,
        is_ignore_cache=is_ignore_cache,
        cache_path=cache_path,
        client=client,
        timeout=timeout,
    )

    if return_raw:
        return res

    return res.response

# %% ../../nbs/jira/routes.ipynb 14
async def get_epic(
    epic_id,
    auth: ja.JiraAuth,
    debug_api: bool = False,
    return_raw: bool = False,
    is_ignore_cache: bool = False,
    cache_path: str = None,
    client: httpx.AsyncClient = None,
    timeout: int = 30,
):
    url = f"https://{auth.instance}/rest/agile/1.0/epic/{epic_id}"

    res = await gd.get_data(
        method="get",
        url=url,
        auth=auth,
        debug_api=debug_api,
        is_ignore_cache=is_ignore_cache,
        cache_path=cache_path,
        client=client,
        timeout=timeout,
    )

    if return_raw:
        return res

    return res.response

# %% ../../nbs/jira/routes.ipynb 18
async def get_issue_by_id(
    auth: ja.JiraAuth,
    issue_id: int,
    debug_api: bool = False,
    return_raw: bool = False,
    is_ignore_cache: bool = False,
    cache_path: str = None,
    client: httpx.AsyncClient = None,
    timeout: int = 30,
):
    url = f"https://{auth.instance}/rest/api/2/issue/{issue_id}"

    res = await gd.get_data(
        method="get",
        url=url,
        auth=auth,
        debug_api=debug_api,
        is_ignore_cache=is_ignore_cache,
        cache_path=cache_path,
        client=client,
        timeout=timeout,
    )

    if return_raw:
        return res

    return res.response

# %% ../../nbs/jira/routes.ipynb 19
async def get_board_issues(
    auth: ja.JiraAuth,
    board_id: int,
    params: dict = None,
    debug_api: bool = False,
    debug_loop: bool = False,
    return_raw: bool = False,
    is_ignore_cache: bool = False,
    cache_path: str = None,
    client: httpx.AsyncClient = None,
    timeout: int = 30,
):
    url = f"https://{auth.instance}/rest/agile/1.0/board/{board_id}/issue"

    def arr_fn(res):
        return res.response["issues"]

    res = await gd.looper(
        method="get",
        url=url,
        auth=auth,
        params=params,
        offset_params={"offset": "startAt", "limit": "maxResults"},
        offset_params_is_header=True,
        debug_api=debug_api,
        debug_loop=debug_loop,
        arr_fn=arr_fn,
        is_ignore_cache=is_ignore_cache,
        cache_path=cache_path,
        client=client,
        timeout=timeout,
    )

    if return_raw:
        return res

    return res.response

# %% ../../nbs/jira/routes.ipynb 20
async def get_issue_links(
    auth: ja.JiraAuth,
    issue_id: int,
    debug_api: bool = False,
    return_raw: bool = False,
    is_ignore_cache: bool = False,
    cache_path: str = None,
    client: httpx.AsyncClient = None,
    timeout: int = 30,
):
    # url = f'https://{auth.instance}/rest/api/2/issueLink'
    url = f"https://{auth.instance}/rest/api/2/issue/{issue_id}/remotelink"

    res = await gd.get_data(
        method="get",
        url=url,
        auth=auth,
        debug_api=debug_api,
        is_ignore_cache=is_ignore_cache,
        cache_path=cache_path,
        client=client,
        timeout=timeout,
    )

    if return_raw:
        return res

    return res.response

# %% ../../nbs/jira/routes.ipynb 21
async def get_issue_comments(
    issue_id,
    auth: ja.JiraAuth,
    debug_api: bool = False,
    return_raw: bool = False,
    is_ignore_cache: bool = False,
    cache_path: str = None,
    client: httpx.AsyncClient = None,
    timeout: int = 30,
):
    url = (
        f"https://{auth.instance}/rest/api/2/issue/{issue_id}/comment?orderBy=-created&maxResults=0&expand=renderedBody"
        if auth.is_datacenter
        else f"https://{auth.instance}/rest/api/2/issue/{issue_id}/comment?orderBy=-created&maxResults=0&expand=renderedBody"
    )

    def arr_fn(res):
        return res.response["comments"]

    res = await gd.looper(
        method="get",
        url=url,
        auth=auth,
        arr_fn=arr_fn,
        offset_params={"offset": "startAt", "limit": "maxResults"},
        offset_params_is_header=True,
        debug_api=debug_api,
        is_ignore_cache=is_ignore_cache,
        cache_path=cache_path,
        client=client,
        tmeout=timeout,
    )

    if return_raw:
        return res

    return res.response



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/utils/Image.ipynb.

# %% auto 0
__all__ = ['isBase64', 'handle_string_to_bytes_and_decode', 'handle_string_to_bytes_and_encode', 'to_bytes', 'crop_square',
           'are_same_image']

# %% ../../nbs/utils/Image.ipynb 2
from PIL.Image import Image

# %% ../../nbs/utils/Image.ipynb 3
from typing import Union
import types
import PIL
import numpy as np

import os
import io
import base64

from nbdev.showdoc import patch_to

# %% ../../nbs/utils/Image.ipynb 4
def isBase64(s):
    try:
        return base64.b64encode(base64.b64decode(s)) == s
    except Exception:
        return False


def handle_string_to_bytes_and_decode(data: Union[str, bytes]):
    if isinstance(data, str):
        data = bytes(data)

    if isBase64(data):
        data = base64.b64decode(data)

    return data


def handle_string_to_bytes_and_encode(data: Union[str, bytes]):
    if isinstance(data, str):
        data = bytes(data)

    if not isBase64(data):
        data = base64.b64encode(data)

    return data

# %% ../../nbs/utils/Image.ipynb 5
def to_bytes(self) -> bytes:
    byte_arr = io.BytesIO()

    if not self.area:
        self.area = self

    self.area.save(byte_arr, format=self.format)

    self.data = byte_arr.getvalue()

    return self.data


def crop_square(self):
    width, height = self.size  # Get dimensions

    new_edge = min(width, height)

    left = (width - new_edge) / 2
    top = (height - new_edge) / 2
    right = (width + new_edge) / 2
    bottom = (height + new_edge) / 2

    # Crop the center of the image
    self.area = self.crop((left, top, right, bottom))

    self.to_bytes()

    return self.area

# %% ../../nbs/utils/Image.ipynb 6
@patch_to(Image, cls_method=True)
def from_image_file(cls, image_path: str) -> Image:
    if not os.path.exists(image_path):
        raise FileNotFoundError(image_path)

    with open(image_path, "rb") as file:
        data = file.read()

    data = handle_string_to_bytes_and_decode(data)

    im = PIL.Image.open(io.BytesIO(data))

    im.to_bytes = types.MethodType(to_bytes, im)
    im.crop_square = types.MethodType(crop_square, im)

    return im

# %% ../../nbs/utils/Image.ipynb 9
@patch_to(Image, cls_method=True)
def from_bytestr(cls, data: Union[str, bytes]) -> Image:
    data = handle_string_to_bytes_and_decode(data)

    im = PIL.Image.open(io.BytesIO(data))

    im.to_bytes = types.MethodType(to_bytes, im)
    im.crop_square = types.MethodType(crop_square, im)

    return im

# %% ../../nbs/utils/Image.ipynb 11
def are_same_image(image1, image2, threshold=0, return_difference: bool = False):
    try:
        img_chop = PIL.ImageChops.difference(image1, image2)

        if return_difference:
            return img_chop

        if np.sum(np.array(img_chop.getdata())) == threshold:
            return True
        return False

    except ValueError as e:
        print(e)
        return False



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/google/GSheet.ipynb.

# %% auto 0
__all__ = ['get_spreadsheet_by_id', 'get_sheets', 'get_sheet_values_df', 'find_row', 'append_rows', 'insert_rows', 'replace_row',
           'GSheet', 'GSpreadsheet']

# %% ../../nbs/google/GSheet.ipynb 1
from dataclasses import dataclass, field
from typing import Any, List, Tuple

import pandas as pd

import domolibrary_extensions.google.GAuth as ga

# %% ../../nbs/google/GSheet.ipynb 4
def get_spreadsheet_by_id(spreadsheet_id, service):
    return (
        service.spreadsheets()
        # .values()
        .get(spreadsheetId=spreadsheet_id).execute()
    )


def get_sheets(spreadsheet_id=None, service=None, spreadsheet=None):
    spreadsheet = (
        spreadsheet
        or service.spreadsheets().get(spreadsheetId=spreadsheet_id).execute()
    )

    return spreadsheet.get("sheets")

# %% ../../nbs/google/GSheet.ipynb 6
def _data_to_df(data):
    rows = data["values"]
    cols = data["values"].pop(0)

    return pd.DataFrame(columns=cols, data=rows)


def get_sheet_values_df(sheet_title, spreadsheet_id, service, return_raw: bool = False):
    data = (
        service.spreadsheets()
        .values()
        .get(spreadsheetId=spreadsheet_id, range=sheet_title)
        .execute()
    )

    if return_raw:
        return data

    return _data_to_df(data)


def find_row(
    sheet_title, spreadsheet_id, service, col_idx, search_val
) -> Tuple[
    int, List[Any]
]:  # tuple with index of row and row value, returns (-1, None) if row not found in data

    data = get_sheet_values_df(
        sheet_title=sheet_title,
        spreadsheet_id=spreadsheet_id,
        service=service,
        return_raw=True,
    )["values"]

    return next(
        ((idx, ele) for idx, ele in enumerate(data) if ele[col_idx] == search_val),
        (-1, None),
    )

# %% ../../nbs/google/GSheet.ipynb 8
def _to_excel(data_ls, delimiter):
    def _convert_rows(row_ls, delimiter):
        return delimiter.join([str(val) for val in row_ls])

    if not isinstance(data_ls[0], list):
        return _convert_rows(data_ls, delimiter)

    return "\n".join([_convert_rows(row_ls, delimiter) for row_ls in data_ls])

# %% ../../nbs/google/GSheet.ipynb 9
def append_rows(
    service,
    spreadsheet_id,
    sheet_title,
    data_ls: List[List[Any]],
    input_option="USER_ENTERED",
):
    return (
        service.spreadsheets()
        .values()
        .append(
            spreadsheetId=spreadsheet_id,
            range=sheet_title,
            body={"majorDimension": "ROWS", "values": data_ls},
            valueInputOption=input_option,
        )
        .execute()
    )


def insert_rows(
    sheet_id,
    service,
    spreadsheet_id,
    insert_row_idx,
    data_ls: List[Any],
    delimiter=";",
):
    batch_update_spreadsheet_request_body = {
        "requests": [
            {
                "insertRange": {
                    "range": {
                        "sheetId": sheet_id,
                        "startRowIndex": insert_row_idx,
                        "endRowIndex": insert_row_idx + len(data_ls),
                    },
                    "shiftDimension": "ROWS",
                }
            },
            {
                "pasteData": {
                    "data": _to_excel(data_ls, delimiter),
                    "type": "PASTE_NORMAL",
                    "delimiter": delimiter,
                    "coordinate": {"sheetId": sheet_id, "rowIndex": insert_row_idx},
                }
            },
        ]
    }

    request = service.spreadsheets().batchUpdate(
        spreadsheetId=spreadsheet_id, body=batch_update_spreadsheet_request_body
    )
    return request.execute()


def replace_row(
    sheet_id,
    spreadsheet_id,
    service,
    row_idx,
    row_ls,
    delimiter=";",
):
    batch_update_spreadsheet_request_body = {
        "requests": [
            {
                "pasteData": {
                    "data": _to_excel(row_ls, delimiter),
                    "type": "PASTE_NORMAL",
                    "delimiter": delimiter,
                    "coordinate": {"sheetId": sheet_id, "rowIndex": row_idx},
                }
            }
        ]
    }

    request = service.spreadsheets().batchUpdate(
        spreadsheetId=spreadsheet_id, body=batch_update_spreadsheet_request_body
    )
    return request.execute()

# %% ../../nbs/google/GSheet.ipynb 12
@dataclass
class GSheet:
    id: int
    title: str
    index: int

    service: Any
    spreadsheet_id: str

    data: List[List[Any]] = None
    df: pd.DataFrame = None

    @classmethod
    def _from_json(cls, obj, service, spreadsheet_id):
        obj = obj["properties"]
        return cls(
            spreadsheet_id=spreadsheet_id,
            service=service,
            id=obj["sheetId"],
            title=obj["title"],
            index=obj["index"],
        )

    def get_data(self, return_raw: bool = False) -> pd.DataFrame:
        self.data = get_sheet_values_df(
            sheet_title=self.title,
            spreadsheet_id=self.spreadsheet_id,
            service=self.service,
            return_raw=True,
        )

        if return_raw:
            return self.data

        self.df = _data_to_df(self.data)
        return self.df

    def append_rows(self, data_ls: List[List[Any]], input_option="USER_ENTERED"):
        append_rows(
            service=self.service,
            spreadsheet_id=self.spreadsheet_id,
            sheet_title=self.title,
            data_ls=data_ls,
            input_option=input_option,
        )

        self.get_data()

    def _find_row(self, col_idx, search_val):
        return find_row(
            sheet_title=self.title,
            spreadsheet_id=self.spreadsheet_id,
            service=self.service,
            col_idx=col_idx,
            search_val=search_val,
        )

    def _replace_row(self, row_idx, row_ls, delimiter=";"):
        return replace_row(
            sheet_id=self.id,
            spreadsheet_id=self.spreadsheet_id,
            service=self.service,
            row_idx=row_idx,
            row_ls=row_ls,
            delimiter=delimiter,
        )

    def update_row(self, col_idx, search_val, new_row_ls, delimiter=";"):
        row_idx = self._find_row(col_idx=col_idx, search_val=search_val)[0]

        if row_idx == -1:
            self.append_rows([new_row_ls])

        else:
            self._replace_row(row_idx, new_row_ls, delimiter)

        return self.get_data()


@dataclass
class GSpreadsheet:
    id: str
    spreadsheet: Any = field(repr=False)
    service: Any = field(repr=False)

    auth: Any = field(default=None, repr=False)

    gsheets: List[dict] = None

    def __post_init__(self):
        self.get_sheets()

    @classmethod
    def get_by_id(
        cls, spreadsheet_id, service=None, auth=None, return_raw: bool = False
    ):

        require_base_url = "https://sheets.googleapis.com/"

        if not auth and not service:
            raise ga.GAuth_Required()

        if not service or service._baseUrl != require_base_url:
            if not auth:
                raise ga.GAuth_Required()

            service = auth.generate_service("sheets", "v4")

        spreadsheet = get_spreadsheet_by_id(
            service=service, spreadsheet_id=spreadsheet_id
        )

        if return_raw:
            return spreadsheet

        return cls(
            auth=auth,
            service=service,
            id=spreadsheet_id,
            spreadsheet=spreadsheet,
        )

    def get_sheets(self, return_raw: bool = False):
        sheets = get_sheets(spreadsheet=self.spreadsheet)

        if return_raw:
            return sheets

        self.gsheets = [
            GSheet._from_json(obj, service=self.service, spreadsheet_id=self.id)
            for obj in sheets
        ]
        return self.gsheets
